
from io import StringIO


BASE58_CHARS = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'

def human_readable_upper_part_map(string: str, backward: bool):
    ''' 为同一字符串序列的不同大小写形式生成不碰撞的字符串。以便在大小写不敏感的系统中存储同一字符串的不同形式。
        
        找到字符串中所有的 ASCII 大写字母，并返回一个能表示他们绝对位置的字符串。
        其中每个非相邻的大写字母之间用数字表示相隔的字符数。

        params: backward: 可以表示是正着看还是倒着看。
        
        NOTE: 在我们的用例中，我们 backward = True ，这样产生的 upper_part 就不太像 BV 号或者类似的编号，以免 upper_part 污染全文搜索。

        例如：
        backward = False
            BV1HP411D7Rj -> BV1HP3D1R （长得像 bvid ）
        backward = True
            BV1HP411D7Rj -> 1R1D3PH1VB
    '''

    assert backward

    if backward:
        string = string[::-1]

    result = StringIO()
    steps = 0
    for char in string:
        if char.isascii() and char.isupper():
            if steps == 0:
                result.write(char)
            else:
                result.write(f'{steps}{char}')
            steps = 0
        else:
            steps += 1

    return result.getvalue()


def is_bvid(string: str):
    ''' 判断一个字符串是否是 Base58 的 bv 号 '''
    if not string.startswith('BV'):
        return False
    for char in string:
        if char not in BASE58_CHARS:
            return False
    
    return True



import json
import os
from pathlib import Path, PurePath
import time
from typing import List
from urllib.parse import urlparse
from internetarchive import get_item
from requests import Response
from rich import print
from pathlib import Path
from shutil import rmtree
from biliarchiver.i18n import _

from biliarchiver.exception import (
    VideosBasePathNotFoundError,
    VideosNotFinishedDownloadError,
)

from biliarchiver.utils.identifier import human_readable_upper_part_map
from biliarchiver.config import BILIBILI_IDENTIFIER_PERFIX, config
from biliarchiver.utils.dirLock import UploadLock, AlreadyRunningError
from biliarchiver.utils.xml_chars import xml_chars_legalize
from biliarchiver.version import BILI_ARCHIVER_VERSION
from biliarchiver.i18n import _


def upload_bvid(
    bvid: str,
    *,
    update_existing: bool = False,
    collection: str,
    delete_after_upload: bool = False,
):
    try:
        lock_dir = config.storage_home_dir / ".locks" / bvid
        lock_dir.mkdir(parents=True, exist_ok=True)
        with UploadLock(lock_dir):  # type: ignore
            _upload_bvid(
                bvid,
                update_existing=update_existing,
                collection=collection,
                delete_after_upload=delete_after_upload,
            )
    except AlreadyRunningError:
        print(_("已经有一个上传 {} 的进程在运行，跳过".format(bvid)))
    except VideosBasePathNotFoundError:
        print(_("没有找到 {} 对应的文件夹。可能是因已存在 IA item 而跳过了下载，或者你传入了错误的 bvid".format(bvid)))
    except VideosNotFinishedDownloadError:
        print(_("{} 的视频还没有下载完成，跳过".format(bvid)))
    except Exception as e:
        print(_("上传 {} 时出错：".format(bvid)))
        raise e


def _upload_bvid(
    bvid: str,
    *,
    update_existing: bool = False,
    collection: str,
    delete_after_upload: bool = False,
):
    access_key, secret_key = read_ia_keys(config.ia_key_file)

    # identifier format: BiliBili-{bvid}_p{pid}-{upper_part}
    upper_part = human_readable_upper_part_map(string=bvid, backward=True)
    OLD_videos_basepath: Path = config.storage_home_dir / "videos" / bvid
    videos_basepath: Path = config.storage_home_dir / "videos" / f"{bvid}-{upper_part}"

    if os.path.exists(OLD_videos_basepath):
        print(f"检测到旧的视频主目录 {OLD_videos_basepath}，将其重命名为 {videos_basepath}...")
        os.rename(OLD_videos_basepath, videos_basepath)

    if not os.path.exists(videos_basepath):
        raise VideosBasePathNotFoundError(f"{videos_basepath}")
    if not (videos_basepath / "_all_downloaded.mark").exists():
        raise VideosNotFinishedDownloadError(f"{videos_basepath}")

    local_identifiers = [f.name for f in videos_basepath.iterdir() if f.is_dir()]
    for local_identifier in local_identifiers:
        remote_identifier = f"{local_identifier}-{upper_part}"
        if (
            os.path.exists(f"{videos_basepath}/{local_identifier}/_uploaded.mark")
            and not update_existing
        ):
            print(
                _("{} => {} 已经上传过了(_uploaded.mark)").format(
                    local_identifier, remote_identifier
                )
            )
            continue
        if local_identifier.startswith("_"):
            print(_("跳过带 _ 前缀的 local_identifier: {}").format(local_identifier))
            continue
        if not local_identifier.startswith(BILIBILI_IDENTIFIER_PERFIX):
            print(
                _("{} 不是以 {} 开头的正确 local_identifier").format(
                    local_identifier, BILIBILI_IDENTIFIER_PERFIX
                )
            )
            continue
        if not os.path.exists(f"{videos_basepath}/{local_identifier}/_downloaded.mark"):
            print(f"{local_identifier} " + _("没有下载完成"))
            continue

        pid = local_identifier.split("_")[-1][1:]
        file_basename = local_identifier[len(BILIBILI_IDENTIFIER_PERFIX) + 1 :]

        print(f"=== {_('开始上传')} {local_identifier} => {remote_identifier} ===")
        item = get_item(remote_identifier)
        if item.exists and not update_existing:
            print(f"item {remote_identifier} {_('已存在')} (item.exists)")
            if item.metadata.get("upload-state") == "uploaded":
                print(f"{remote_identifier} {_('已经上传过了，跳过')} (item.metadata.uploaded)")
                with open(
                    f"{videos_basepath}/{local_identifier}/_uploaded.mark",
                    "w",
                    encoding="utf-8",
                ) as f:
                    f.write("")
                continue
        with open(
            f"{videos_basepath}/{local_identifier}/extra/{file_basename}.info.json",
            "r",
            encoding="utf-8",
        ) as f:
            bv_info = json.load(f)

        cover_url: str = bv_info["data"]["View"]["pic"]
        cover_suffix = PurePath(urlparse(cover_url).path).suffix

        filedict = {}  # "remote filename": "local filename"
        for filename in os.listdir(f"{videos_basepath}/{local_identifier}/extra"):
            file = f"{videos_basepath}/{local_identifier}/extra/{filename}"
            if os.path.isfile(file):
                if file.startswith("_"):
                    continue
                filedict[filename] = file

                # 复制一份 cover 作为 itemimage
                if filename == f"{bvid}_p{pid}{cover_suffix}":
                    filedict[f"{bvid}_p{pid}_itemimage{cover_suffix}"] = file

        for filename in os.listdir(f"{videos_basepath}/{local_identifier}"):
            file = f"{videos_basepath}/{local_identifier}/{filename}"
            if os.path.isfile(file):
                if os.path.basename(file).startswith("_"):
                    continue
                if not os.path.isfile(file):
                    continue
                filedict[filename] = file

        assert (f"{file_basename}.mp4" in filedict) or (
            f"{file_basename}.flv" in filedict
        )

        # IA 去重
        for file_in_item in item.files:
            if file_in_item["name"] in filedict:
                filedict.pop(file_in_item["name"])
                print(
                    f"File {file_in_item['name']} already exists in {remote_identifier}."
                )

        # with open(f'{videos_basepath}/_videos_info.json', 'r', encoding='utf-8') as f:
        #     videos_info = json.load(f)

        tags = ["BiliBili", "video"]
        for tag in bv_info["data"]["Tags"]:
            tags.append(tag["tag_name"])
        pubdate = bv_info["data"]["View"]["pubdate"]
        cid = None
        p_part = None
        for page in bv_info["data"]["View"]["pages"]:
            if page["page"] == int(pid):
                cid = page["cid"]
                p_part = page["part"]
                break

        assert cid is not None
        assert p_part is not None

        aid = bv_info["data"]["View"]["aid"]
        owner_mid = bv_info["data"]["View"]["owner"]["mid"]
        owner_creator: str = bv_info["data"]["View"]["owner"]["name"]  # UP 主

        mids: List[int] = [owner_mid]
        creators: List[str] = [owner_creator]
        if bv_info["data"]["View"].get("staff") is not None:
            mids = []  # owner_mid 在 staff 也有
            creators = []
            for staff in bv_info["data"]["View"]["staff"]:
                mids.append(staff["mid"]) if staff["mid"] not in mids else None
                creators.append(staff["name"]) if staff[
                    "name"
                ] not in creators else None
        external_identifier = [
            f"urn:bilibili:video:aid:{aid}",
            f"urn:bilibili:video:bvid:{bvid}",
            f"urn:bilibili:video:cid:{cid}",
        ]
        for mid in mids:
            external_identifier.append(f"urn:bilibili:video:mid:{mid}")

        md = {
            "mediatype": "movies",
            "collection": collection,
            "title": bv_info["data"]["View"]["title"] + f" P{pid} " + p_part,
            "description": remote_identifier + " uploading...",
            "creator": creators
            if len(creators) > 1
            else owner_creator,  # type: list[str] | str
            # UTC time
            "date": time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(pubdate)),
            # 'aid': aid,
            # 'bvid': bvid,
            # 'cid': cid,
            # 'mid': mid,
            "external-identifier": external_identifier,
            "subject": "; ".join(
                tags
            ),  # Keywords should be separated by ; but it doesn't matter much; the alternative is to set one per field with subject[0], subject[1], ...
            "upload-state": "uploading",
            "originalurl": f"https://www.bilibili.com/video/{bvid}/?p={pid}",
            "scanner": f"biliarchiver v{BILI_ARCHIVER_VERSION} (dev)",
        }

        print(filedict)
        print(md)

        # remove XML illegal characters
        _md_before = hash(json.dumps(md))
        md = xml_chars_legalize(obj=md)
        assert isinstance(md, dict)
        if hash(json.dumps(md)) != _md_before:
            print("Removed XML illegal characters from metadata, cleaned metadata:")
            print(md)

        if filedict:
            r = item.upload(
                files=filedict,
                metadata=md,
                access_key=access_key,
                secret_key=secret_key,
                verbose=True,
                queue_derive=True,
                retries=5,
            )

        tries = 100
        item = get_item(remote_identifier)  # refresh item
        while not item.exists and tries > 0:
            print(f"Waiting for item to be created ({tries})  ...", end="\r")
            time.sleep(30)
            item = get_item(remote_identifier)
            tries -= 1

        new_md = {}
        if item.metadata.get("upload-state") != "uploaded":
            new_md["upload-state"] = "uploaded"
        if item.metadata.get("creator") != md["creator"]:
            new_md["creator"] = md["creator"]
        if item.metadata.get("description", "") != bv_info["data"]["View"]["desc"]:
            new_md["description"] = bv_info["data"]["View"]["desc"]
        if item.metadata.get("scanner") != md["scanner"]:
            new_md["scanner"] = md["scanner"]
        if item.metadata.get("external-identifier") != md["external-identifier"]:
            new_md["external-identifier"] = md["external-identifier"]
        if new_md:
            print("Updating metadata:")
            print(new_md)

            # remove XML illegal characters
            _md_before = hash(json.dumps(new_md))
            new_md = xml_chars_legalize(obj=new_md)
            assert isinstance(new_md, dict)
            if hash(json.dumps(new_md)) != _md_before:
                print("Removed XML illegal characters from metadata, cleaned metadata:")
                print(new_md)

            r = item.modify_metadata(
                metadata=new_md,
                access_key=access_key,
                secret_key=secret_key,
            )
            assert isinstance(r, Response)
            r.raise_for_status()
        with open(
            f"{videos_basepath}/{local_identifier}/_uploaded.mark",
            "w",
            encoding="utf-8",
        ) as f:
            f.write("")
        print(f"==== {remote_identifier} {_('上传完成')} ====")

    if delete_after_upload and len(local_identifiers) > 0:
        try:
            for local_identifier in local_identifiers:
                rmtree(f"{videos_basepath}/{local_identifier}")
            print(
                "[yellow]"
                + _("已删除视频文件夹 {}").format(", ".join(local_identifiers))
                + "[/yellow]"
            )
        except Exception as e:
            print(e)


def read_ia_keys(keysfile):
    """Return: tuple(`access_key`, `secret_key`)"""
    with open(keysfile, "r", encoding="utf-8") as f:
        key_lines = f.readlines()

    access_key = key_lines[0].strip()
    secret_key = key_lines[1].strip()

    return access_key, secret_key



import gettext
import locale
from pathlib import Path
import warnings

default_lang, default_enc = locale.getdefaultlocale()
default_lang = default_lang or "en"
languages = ["en"] if not default_lang.lower().startswith("zh") else ["zh_CN"]
appname = "biliarchiver"

localedir = Path(__file__).parent / "locales"
if not localedir.exists():
    warnings.warn("Locales directory not found, i18n will not work.", RuntimeWarning)

i18n = gettext.translation(
    appname, localedir=localedir, fallback=True, languages=languages
)

_ = i18n.gettext
ngettext = i18n.ngettext



import click
from biliarchiver.i18n import _
from biliarchiver.cli_tools.up_command import up
from biliarchiver.cli_tools.down_command import down
from biliarchiver.cli_tools.get_command import get
from biliarchiver.cli_tools.conf_command import config
from biliarchiver.version import BILI_ARCHIVER_VERSION


class HelpCommand(click.Group):
    def format_help(self, ctx, formatter):
        version_info = click.style(
            f"[ biliarchiver {BILI_ARCHIVER_VERSION} ]",
            fg="black",
            bg="white",
            bold=True,
            blink=True,
        )
        click.echo(
            f"""
                <_-^-_>
       <%&&&&&&&&&&&&&&&&&&&&%*>
 <&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&>      > __      __   __   __       <
  /<&&&>     <&&&>/      .<&&&>      <&&&>  >      / /     /_/  / /  /_/ <
  (<&&&>,(/(/<&&&>%(/(/(/(<&&&>(/(// <&&&>  >     / /_    __   / /  __   <
  %<&&&>((   <&&&#>      <#&&&>   (((<&&&(> >    / __ \  / /  / /  / /   <
  <&&&&>/*   <&&&#>,    .<#&&&>   .//<&&&(> >   / /_/ / / /  / /  / /   __      __   <
  <&&&&>(* ,(<&&&#>%    %<#&&&>%(  ((<&&&(> >  /_.___/ /_/  /_/  /_/   / /     /_/   <
  <&&&&>/*  .<&&&#>      <#&&&>%   (/<&&&(> >    ____ _  ____  _____  / /_    __  _   __  ___    ____ <
  (<&&&>(*   <&&&>#,,-w-,#<&&&>    ((<&&&(> >   / __ `/ / __/ / ___/ / __ \  / / | | / / / _ \  / __/ <
  (<&&&>/*   <&&&>,       <&&&>   .//<&&&)> >  / /_/ / / /   / /__  / / / / / /  | |/ / /  __/ / / <
   <&&&>((/((<&&&>((((((((<&&&>(((/((<&&&>  >  \__,_/ /_/    \___/ /_/ /_/ /_/   |___/  \___/ /_/  <
   <&&&>  ,,(<&&&>        <&&&>(,,   <&&&>
  <&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&>               {version_info}
<&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&><""".replace(
                "<", "\u001b[0m"
            ).replace(
                ">", "\u001b[96m"
            ),
        )
        click.echo(super().format_help(ctx, formatter))


@click.group(cls=HelpCommand)
def biliarchiver():
    pass


@biliarchiver.command(help=click.style(_("初始化所需目录"), fg="cyan"))
def init():
    import os
    import pathlib

    biliarchiver_home = pathlib.Path.cwd() / "biliarchiver.home"
    bilibili_archive_dir = pathlib.Path.cwd() / "bilibili_archive_dir"

    # 猫猫创成文件夹了
    if biliarchiver_home.exists() and not biliarchiver_home.is_file():
        try:
            os.removedirs(biliarchiver_home)
        except Exception as e:
            print(e) # 不是啥问题，反正路径还存在

    biliarchiver_home.touch(exist_ok=True)
    bilibili_archive_dir.mkdir(exist_ok=True)


biliarchiver.add_command(up)
biliarchiver.add_command(down)
biliarchiver.add_command(get)
biliarchiver.add_command(config)


@biliarchiver.command(help=click.style(_("配置账号信息"), fg="cyan"))
def auth():
    click.echo(click.style("Bilibili", bg="yellow"))
    click.echo(_("登录后将哔哩哔哩的 cookies 复制到 `config.json` 指定的文件（默认为 `~/.cookies.txt`）中。"))
    click.echo("")
    click.echo(click.style("Internet archive", bg="yellow"))
    click.echo(_("前往 https://archive.org/account/s3.php 获取 Access Key 和 Secret Key。"))
    click.echo(_("""将它们放在 `config.json` 指定的文件（默认为 `~/.bili_ia_keys.txt`）中，两者由换行符分隔。"""))
    


if __name__ == "__main__":
    biliarchiver()



from pathlib import Path
from biliarchiver.utils.identifier import is_bvid
from biliarchiver.i18n import _


def read_bvids(bvids: str) -> list[str]:
    bvids_list = None

    file = Path(bvids)
    if file.exists() and file.is_file():
        with open(file, "r", encoding="utf-8") as f:
            bvids_list = f.read().split()
    else:
        bvids_list = bvids.split()

    del bvids

    for bvid in bvids_list:
        assert is_bvid(bvid), _("bvid {} 不合法").format(bvid)

    assert bvids_list is not None and len(bvids_list) > 0, _("bvids 为空")

    return bvids_list



import asyncio
import os
from pathlib import Path
from typing import List, Optional, Union

from bilix.sites.bilibili.downloader import DownloaderBilibili
from httpx import AsyncClient, Client, TransportError
from rich.traceback import install

from biliarchiver.archive_bvid import archive_bvid
from biliarchiver.config import config
from biliarchiver.config import BILIBILI_IDENTIFIER_PERFIX
from biliarchiver.utils.http_patch import HttpOnlyCookie_Handler
from biliarchiver.utils.version_check import check_outdated_version
from biliarchiver.utils.storage import get_free_space
from biliarchiver.utils.identifier import human_readable_upper_part_map
from biliarchiver.utils.ffmpeg import check_ffmpeg
from biliarchiver.version import BILI_ARCHIVER_VERSION
from biliarchiver.cli_tools.utils import read_bvids
from biliarchiver.i18n import _

install()


def check_ia_item_exist(client: Client, identifier: str) -> bool:
    cache_dir = config.storage_home_dir / "ia_item_exist_cache"
    # check_ia_item_exist_from_cache_file:
    if (cache_dir / f"{identifier}.mark").exists():
        # print('from cached .mark')
        return True

    def create_item_exist_cache_file(identifier: str) -> Path:
        with open(cache_dir / f"{identifier}.mark", "w", encoding="utf-8") as f:
            f.write("")
        return cache_dir / f"{identifier}.mark"

    params = {
        "identifier": identifier,
        "output": "json",
    }
    # check_identifier.php API 响应快
    r = None
    for _ in range(3):
        try:
            r = client.get(
                "https://archive.org/services/check_identifier.php", params=params
            )
            break
        except TransportError as e:
            print(e, "retrying...")
    assert r is not None
    r.raise_for_status()
    r_json = r.json()
    assert r_json["type"] == "success"
    if r_json["code"] == "available":
        return False
    elif r_json["code"] == "not_available":  # exists
        cache_dir.mkdir(parents=True, exist_ok=True)
        create_item_exist_cache_file(identifier)
        return True
    else:
        raise ValueError(f'Unexpected code: {r_json["code"]}')


def _down(
    bvids: str,
    skip_ia_check: bool,
    from_browser: Optional[str],
    min_free_space_gb: int,
    skip_to: int,
):
    assert check_ffmpeg() is True, _("ffmpeg 未安装")

    bvids_list = read_bvids(bvids)

    check_outdated_version(
        pypi_project="biliarchiver", self_version=BILI_ARCHIVER_VERSION
    )

    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)

    d = DownloaderBilibili(
        hierarchy=True,
        sess_data=None,  # sess_data 将在后面装载 cookies 时装载 # type: ignore
        video_concurrency=config.video_concurrency,
        part_concurrency=config.part_concurrency,
        stream_retry=config.stream_retry,
    )

    # load cookies
    if from_browser is not None:
        update_cookies_from_browser(d.client, from_browser)
    else:
        update_cookies_from_file(d.client, config.cookies_file)
    client = Client(cookies=d.client.cookies, headers=d.client.headers)
    logined = is_login(client)
    if not logined:
        return

    def check_free_space():
        if min_free_space_gb != 0:
            if (
                get_free_space(path=config.storage_home_dir) // 1024 // 1024 // 1024
                <= min_free_space_gb
            ):
                return False  # not pass
        return True  # pass

    d.progress.start()
    sem = asyncio.Semaphore(config.video_concurrency)
    tasks: List[asyncio.Task] = []

    def tasks_check():
        for task in tasks:
            if task.done():
                _task_exception = task.exception()
                if isinstance(_task_exception, BaseException):
                    print(f"任务 {task} 出错，即将异常退出...")
                    for task in tasks:
                        task.cancel()
                    raise _task_exception
                # print(f'任务 {task} 已完成')
                tasks.remove(task)
        if not check_free_space():
            s = _("剩余空间不足 {} GiB").format(min_free_space_gb)
            print(s)
            for task in tasks:
                task.cancel()
            raise RuntimeError(s)

    for index, bvid in enumerate(bvids_list):
        if index < skip_to:
            print(f"跳过 {bvid} ({index+1}/{len(bvids_list)})", end="\r")
            continue
        tasks_check()
        if not skip_ia_check:
            upper_part = human_readable_upper_part_map(string=bvid, backward=True)
            remote_identifier = f"{BILIBILI_IDENTIFIER_PERFIX}-{bvid}_p1-{upper_part}"
            if check_ia_item_exist(client, remote_identifier):
                print(_("IA 上已存在 {}，跳过").format(remote_identifier))
                continue

        upper_part = human_readable_upper_part_map(string=bvid, backward=True)
        videos_basepath: Path = (
            config.storage_home_dir / "videos" / f"{bvid}-{upper_part}"
        )
        if os.path.exists(videos_basepath / "_all_downloaded.mark"):
            print(_("{} 的所有分p都已下载过了").format(bvid))
            continue

        if len(tasks) >= config.video_concurrency:
            loop.run_until_complete(
                asyncio.wait(tasks, return_when=asyncio.FIRST_COMPLETED)
            )
            tasks_check()

        print(f"=== {bvid} ({index+1}/{len(bvids_list)}) ===")

        task = loop.create_task(
            archive_bvid(d, bvid, logined=logined, semaphore=sem),
            name=f"archive_bvid({bvid})",
        )
        tasks.append(task)

    while len(tasks) > 0:
        loop.run_until_complete(
            asyncio.wait(tasks, return_when=asyncio.FIRST_COMPLETED)
        )
        tasks_check()

    print("DONE")


def update_cookies_from_browser(client: AsyncClient, browser: str):
    try:
        import browser_cookie3

        f = getattr(browser_cookie3, browser.lower())
        cookies_to_update = f(domain_name="bilibili.com")
        client.cookies.update(cookies_to_update)
        print(_("从 {} 品尝了 {} 块 cookies").format(browser, len(cookies_to_update)))
    except AttributeError:
        raise AttributeError(f"Invalid Browser {browser}")


def update_cookies_from_file(client: AsyncClient, cookies_path: Union[str, Path]):
    if isinstance(cookies_path, Path):
        cookies_path = cookies_path.expanduser()
    elif isinstance(cookies_path, str):
        cookies_path = Path(cookies_path).expanduser()
    else:
        raise TypeError(f"cookies_path: {type(cookies_path)}")

    assert os.path.exists(cookies_path), _("cookies 文件不存在: {}").format(cookies_path)

    from http.cookiejar import MozillaCookieJar

    cj = MozillaCookieJar()

    with HttpOnlyCookie_Handler(cookies_path):
        cj.load(f"{cookies_path}", ignore_discard=True, ignore_expires=True)
        loadded_cookies = 0
        loadded_keys = []
        for cookie in cj:
            # only load bilibili cookies
            if "bilibili.com" not in cookie.domain:
                continue
            if cookie.name in loadded_keys:
                print(_("跳过重复的 cookies"), end="")
                print(f": {cookie.name}")
                # httpx 不能处理不同域名的同名 cookies，只好硬去重了
                continue
            assert cookie.value is not None
            client.cookies.set(
                cookie.name, cookie.value, domain=cookie.domain, path=cookie.path
            )
            loadded_keys.append(cookie.name)
            loadded_cookies += 1
        print(_("从 {} 品尝了 {} 块 cookies").format(cookies_path, loadded_cookies))
        if loadded_cookies > 100:
            print(_("吃了过多的 cookies，可能导致 httpx.Client 怠工，响应非常缓慢"))

        assert client.cookies.get("SESSDATA") is not None, "SESSDATA 不存在"
        # print(f'SESS_DATA: {client.cookies.get("SESSDATA")}')


def is_login(cilent: Client) -> bool:
    r = cilent.get("https://api.bilibili.com/x/member/web/account")
    r.raise_for_status()
    nav_json = r.json()
    if nav_json["code"] == 0:
        print(_("BiliBili 登录成功，饼干真香。"))
        print(_("NOTICE: 存档过程中请不要在 cookies 的源浏览器访问 B 站，避免 B 站刷新"), end=" ")
        print(_("cookies 导致我们半路下到的视频全是 480P 的优酷土豆级醇享画质。"))
        return True
    print(_("未登录/SESSDATA无效/过期，你这饼干它保真吗？"))
    return False


if __name__ == "__main__":
    raise DeprecationWarning(_("已废弃直接运行此命令，请改用 biliarchiver 命令"))



import requests

from biliarchiver.exception import VersionOutdatedError


def get_latest_version(pypi_project: str):
    '''Returns the latest version of pypi_project.'''
    project_url_pypi = f'https://pypi.org/pypi/{pypi_project}/json'
    
    try:
        response = requests.get(project_url_pypi, timeout=5, headers={'Accept': 'application/json', 'Accept-Encoding': 'gzip'})
    except Exception as e:
        print(f'Warning: Could not get latest version of {pypi_project} from pypi.org. (Exception: {e})')
        return None

    if response.status_code == 200:
        data = response.json()
        latest_version: str = data['info']['version']
        return latest_version
    else:
        print(f'Warning: Could not get latest version of {pypi_project} from pypi.org. HTTP status_code: {response.status_code}')
        return None

def check_outdated_version(pypi_project: str, self_version: str, raise_error: bool = True):
    latest_version = get_latest_version(pypi_project)
    if latest_version is None:
        return
    elif latest_version != self_version:
        print('=' * 47)
        print(f'Warning: You are using an outdated version of {pypi_project} ({self_version}).')
        print(f'         The latest version is {latest_version}.')
        print(f'         You can update {pypi_project} with "pip3 install --upgrade {pypi_project}".')
        print('=' * 47, end='\n\n')
        if raise_error:
            raise VersionOutdatedError(version=self_version)
    else: # latest_version == self_version
        print(f'You are using the latest version of {pypi_project}.')



from io import TextIOWrapper
import click
import os

from biliarchiver.cli_tools.utils import read_bvids
from biliarchiver.i18n import _


DEFAULT_COLLECTION = "opensource_movies"
"""
开放 collection ，任何人均可上传。
通过 biliarchiver 上传的 item 会在24小时内被自动转移到 bilibili_videos collection
"""
"""
An open collection. Anyone can upload.
Items uploaded by biliarchiver will be automatically moved to bilibili_videos collection within 24 hours.
"""

BILIBILI_VIDEOS_COLLECTION = "bilibili_videos"
""" 由 arkiver 管理。bilibili_videos 属于 social-media-video 的子集 """
""" Managed by arkiver. bilibili_videos is a subset of social-media-video """

BILIBILI_VIDEOS_SUB_1_COLLECTION = "bilibili_videos_sub_1"
""" 由 yzqzss 管理。属于 bilibili_videos 的子集 """
""" Managed by yzqzss. A subset of bilibili_videos """


@click.command(help=click.style(_("上传至互联网档案馆"), fg="cyan"))
@click.option("--bvids", "-i", type=click.STRING, default=None, help=_("bvids 列表的文件路径"))
@click.option(
    "--by-storage-home-dir",
    "-a",
    is_flag=True,
    default=False,
    help=_("使用 `$storage_home_dir/videos` 目录下的所有视频"),
)
@click.option("--update-existing", is_flag=True, default=False, help=_("更新已存在的 item"))
@click.option(
    "--collection",
    "-c",
    default=DEFAULT_COLLECTION,
    type=click.Choice(
        [
            DEFAULT_COLLECTION,
            BILIBILI_VIDEOS_COLLECTION,
            BILIBILI_VIDEOS_SUB_1_COLLECTION,
        ]
    ),
    help=_("欲上传至的 collection. (非默认值仅限 collection 管理员使用)")
    + f" [default: {DEFAULT_COLLECTION}]",
)
@click.option(
    "--delete-after-upload",
    "-d",
    is_flag=True,
    default=False,
    help=_("上传后删除视频文件"),
)
def up(
    bvids: TextIOWrapper,
    by_storage_home_dir: bool,
    update_existing: bool,
    collection: str,
    delete_after_upload: bool,
):
    from biliarchiver._biliarchiver_upload_bvid import upload_bvid
    from biliarchiver.config import config

    ids = []

    if by_storage_home_dir:
        for bvid_with_upper_part in os.listdir(config.storage_home_dir / "videos"):
            bvid = bvid_with_upper_part
            if "-" in bvid_with_upper_part:
                bvid = bvid_with_upper_part.split("-")[0]
            ids.append(bvid)
    elif bvids:
        ids = read_bvids(bvids)

    for id in ids:
        upload_bvid(
            id,
            update_existing=update_existing,
            collection=collection,
            delete_after_upload=delete_after_upload,
        )



from biliarchiver.i18n import _


class VideosBasePathNotFoundError(FileNotFoundError):
    def __init__(self, path: str):
        self.path = path

    def __str__(self):
        return f"Videos base path {self.path} not found"


class VideosNotFinishedDownloadError(FileNotFoundError):
    def __init__(self, path: str):
        self.path = path

    def __str__(self):
        return f"Videos not finished download: {self.path}"


class VersionOutdatedError(Exception):
    def __init__(self, version):
        self.version = version

    def __str__(self):
        return _("请更新 biliarchiver。当前版本已过期：") + str(self.version)


class DirNotInitializedError(Exception):
    def __init__(self):
        pass

    def __str__(self):
        return _("先在当前工作目录运行 `biliarchiver init` 以初始化配置")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['biliarchiver', 'biliarchiver.cli_tools', 'biliarchiver.utils']

package_data = \
{'': ['*'], 'biliarchiver': ['locales/*', 'locales/en/LC_MESSAGES/*']}

install_requires = \
['bilix==0.18.4',
 'browser-cookie3>=0.19.1,<0.20.0',
 'click-option-group>=0.5.6,<0.6.0',
 'click>=8.1.6,<9.0.0',
 'danmakuc>=0.3.6,<0.4.0',
 'internetarchive>=3.5.0,<4.0.0']

entry_points = \
{'console_scripts': ['biliarchiver = '
                     'biliarchiver.cli_tools.biliarchiver:biliarchiver']}

setup_kwargs = {
    'name': 'biliarchiver',
    'version': '0.1.4',
    'description': '',
    'long_description': "# biliarchiver\n\n> 基于 bilix 的 BiliBili 存档工具\n\n## Install\n\n```bash\npip install biliarchiver\n```\n\n## Usage\n\n```bash\nbiliarchiver --help\n```\n\nFollow these steps to start archiving:\n\n1. Initialize a new workspace in current working directory:\n```bash\nbiliarchiver init\n```\n2. Provide cookies and tokens following instructions:\n```bash\nbiliarchiver auth\n```\n3. Download videos from BiliBili:\n```bash\nbiliarchiver down --bvids BVXXXXXXXXX\n```\n- This command also accepts a list of BVIDs or path to a file. Details can be found in `biliarchiver down --help`.\n4. Upload videos to Internet Archive:\n```bash\nbiliarchiver up --bvids BVXXXXXXXXX\n```\n- This command also accepts a list of BVIDs or path to a file. Details can be found in `biliarchiver up --help`.\n\n## Develop\n\n### Install\n\nPlease use poetry to install dependencies:\n\n```sh\npoetry install\n```\n\nBuild English locale if necessary. Refer to the last section for details.\n\n### Run\n\n```sh\npoetry run biliarchiver --help\n```\n\n### Lint\n\n```sh\npoetry run ruff check .\n```\n\n### i18n\n\nTo generate and build locales, you need a GNU gettext compatible toolchain. You can install `mingw` and use `sh` to enter a bash shell on Windows.\n\nGenerate `biliarchiver.pot`:\n\n```sh\nfind biliarchiver/ -name '*.py' | xargs xgettext -d base -o biliarchiver/locales/biliarchiver.pot\n```\n\nAdd a new language:\n\n```sh\nmsginit -i biliarchiver/locales/biliarchiver.pot -o en.po -l en\n```\n\nUpdate a language:\n\n```sh\npnpx gpt-po sync --po biliarchiver/locales/en/LC_MESSAGES/biliarchiver.po --pot biliarchiver/locales/biliarchiver.pot\n```\n\n**(Important)** Build a language:\n\n```sh\nmsgfmt biliarchiver/locales/en/LC_MESSAGES/biliarchiver.po -o biliarchiver/locales/en/LC_MESSAGES/biliarchiver.mo\n```\n",
    'author': 'yzqzss',
    'author_email': 'yzqzss@yandex.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
